package com.example.arqdemo;

import com.example.arqdemo.Demo;
import java.io.File;
import javax.inject.Inject;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.StringAsset;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.shrinkwrap.descriptor.api.Descriptors;
import org.jboss.shrinkwrap.descriptor.api.beans10.BeansDescriptor;
import org.jboss.shrinkwrap.descriptor.api.persistence10.PersistenceDescriptor;
import org.jboss.shrinkwrap.resolver.api.DependencyResolvers;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependencyResolver;
import org.junit.*;
import static org.junit.Assert.*;
import org.junit.runner.RunWith;

//
// This is a small demo showing how Arquillian can be used to run tests
// involving the java persistence API (JPA) in a container.
//
@RunWith(Arquillian.class)
public class DemoTest {

    //
    // This method is called by Arquillian to create the test archive. It is
    // not run on the application server; it runs on a client-side JVM.
    //
    // All the classes referenced in the @Deployment method
    // must still be resolvable from the deployed web archive
    // because the class as a whole must be loadable
    // on the application server. That means that if you only use a library
    // within the @Deployment method you must still ensure that ShrinkWrap
    // includes it (or api stubs for it) in your test archive. ShrinkWrap and
    // Arquillian automatically provide thin api stubs for their own APIs,
    // so you don't need to worry about them.
    // 
    @Deployment
    public static WebArchive createDeployment() {
        
        // Use the Maven dependency resolver to import all our dependencies
        // If preferred, you can explicitly specify only the dependencies
        // you want using .artifacts("theGroupId:theArtifactId", ...)
        // or specify artifacts not listed in your pom by adding a :version to
        // the co-ordinates passed to .artifacts(...).
        //
        // See: 
        //  https://community.jboss.org/wiki/FAQHowToIAddMavenArtifactsToMyShrinkWrapArchives
        //
        File[] mvnLibs = DependencyResolvers.use(MavenDependencyResolver.class)
                .loadEffectivePom("pom.xml")
                .importAllDependencies()
                .resolveAsFiles();

        // Load our project's beans.xml
        //
        // We can use the BeansDescriptor to make changes to the beans.xml
        // like substituting @Alternative beans, etc.
        //
        // If you just need an empty beans.xml, you can use:
        //  
        //    Descriptors.create(BeansDescriptor.class);
        //
        // instead.
        //
        // In of ShrinkWrap Descriptors 2.0.0-alpha-1, a bug prevents 
        // the addition of alternatives to an existing <alternatives/> clause.
        // See: 
        //   https://issues.jboss.org/browse/SHRINKDESC-115
        // Check for updates; a later release may have fixed this.
        //
        BeansDescriptor beansXml = Descriptors.importAs(BeansDescriptor.class)
                .from(new File("src/main/webapp/WEB-INF/beans.xml"));

        //
        // Load the project's persistence.xml. As with beans.xml we could alter
        // it to, eg, switch to a testing-only JNDI datasource, but in this example
        // we're just going to copy it.
        //
        PersistenceDescriptor persistenceXml = Descriptors.importAs(PersistenceDescriptor.class)
                .from(new File("src/main/resources/META-INF/persistence.xml"));

        //
        // Now wrap the test class, dependencies, descriptors, and the class(es) being
        // tested into an archive for deployment to the server.
        //
        return ShrinkWrap.create(WebArchive.class)
                .addPackage(Demo.class.getPackage())
                .addAsWebInfResource(new StringAsset(beansXml.exportAsString()), "beans.xml")
                .addAsResource(new StringAsset(persistenceXml.exportAsString()), "META-INF/persistence.xml")
                .addAsLibraries(mvnLibs);
    }
    
    // If beans.xml is included, CDI injection works in unit tests run on the
    // container, so we can use it to get an instance of our demo bean with
    // the EntityManager properly injected.
    //
    @Inject
    private Demo d;
    
    @Test
    public void testEntityManagerInjected() {
        assertNotNull(d);
    }
    
    @Test
    public void testBasicPersistence() {
        assertNull(d.getGarbage(1));
        d.setGarbage(1, "garbage");
        assertEquals(d.getGarbage(1), "garbage");
    }
    
    @Test
    public void testCommonsLang() {
        // Use a method that requires Apache Commons Lang 3 to function
        // just to prove that we've successfully pulled the dependency in.
        d.setGarbageToRandom(2);
        String randomGarbageString = d.getGarbage(2);
        System.err.println("Got random garbage string: " + randomGarbageString);
    }
    
}
